
/***********************************************************************/
/* nbase_memalloc.c -- A few simple functions related to memory        *
 * allocation.  Right now they are just safe_ versions of well known   *
 * calls like malloc that quit if the allocation fails (so you don't   *
 * have to have a bunch of ugly checks in your code.                   *
 * These were written by fyodor@insecure.org .                         *
 *                                                                     *
 ***********************************************************************
 *                                                                     *
 *  Many of the files contained in libnbase are compatability          *
 *  functions written by others.  License conditions for those files   *
 *  may vary and is generally included at the top of the files.   Be   *
 *  sure to read that information before you redistribute or           *
 *  incorporate parts of those files into your software.               *
 *                                                                     *   
 *  Source is provided to this software because we believe users have  *
 *  a right to know exactly what a program is going to do before they  *
 *  run it.  This also allows you to audit the software for security   *
 *  holes (none have been found so far).                               *
 *                                                                     *
 *  Source code also allows you to port libnbase to new platforms, fix *
 *  bugs, and add new features.  You are highly encouraged to send     *
 *  your changes to fyodor@insecure.org for possible incorporation     *
 *  into the main distribution.  By sending these changes to Fyodor or *
 *  one the insecure.org development mailing lists, it is assumed that *
 *  you are offering Fyodor the unlimited, non-exclusive right to      *
 *  reuse, modify, and relicense the code.  This is important because  *
 *  the inability to relicense code has caused devastating problems    *
 *  for other Free Software projects (such as KDE and NASM).  Nmap     *
 *  will always be available Open Source.  If you wish to specify      *
 *  special license conditions of your contributions, just say so      *
 *  when you send them.                                                *
 *                                                                     *
 *  This program is distributed in the hope that it will be useful,    *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.               *
 *                                                                     *
 ***********************************************************************/

/* $Id: nbase_memalloc.c,v 1.1 2003/08/16 04:07:02 fyodor Exp $ */

#include "nbase.h"
#include <stdio.h>

static void fatal(char *fmt, ...) __attribute__ ((format (printf, 1, 2)));

static void fatal(char *fmt, ...) 
{
va_list ap;
va_start(ap, fmt);
fflush(stdout);
vfprintf(stderr, fmt, ap);
fprintf(stderr, "\nQUITTING!\n");
va_end(ap);
exit(1);
}

void *safe_malloc(int size)
{
  void *mymem;
  if (size < 0)
    fatal("Tried to malloc negative amount of memory!!!");
  mymem = malloc(size);
  if (mymem == NULL)
    fatal("Malloc Failed! Probably out of space.");
  //  printf("Called safe_malloc(%d) -- returning %lX\n", size, (unsigned long) mymem);
  return mymem;
}

void *safe_realloc(void *ptr, size_t size)
{
  void *mymem;
  if (size < 0)
    fatal("Tried to realloc negative amount of memory!!!");
  mymem = realloc(ptr, size);
  if (mymem == NULL)
    fatal("Realloc Failed! Probably out of space.");
  //  printf("Called safe_malloc(%d) -- returning %lX\n", size, (unsigned long) mymem);
  return mymem;
}

/* Zero-initializing version of safe_malloc */
void *safe_zalloc(int size)
{
  void *mymem;
  if (size < 0)
    fatal("Tried to malloc negative amount of memory!!!");
  mymem = calloc(1, size);
  if (mymem == NULL)
    fatal("Malloc Failed! Probably out of space.");
  //  printf("Called safe_zalloc(%d) -- returning %lX\n", size, (unsigned long) mymem);
  return mymem;
}
